/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this file,
 * You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"

[scriptable, uuid(5805ac8b-7cbe-4fbd-97ad-d3ae8cd29f79)]
interface nsIFxAccountsUIGlue : nsISupports
{
  // Returns a Promise.
  jsval signInFlow();
};
