/*
 * DO NOT EDIT.  THIS FILE IS GENERATED FROM ../../../dist/idl/nsIHttpDataUsage.idl
 */

#ifndef __gen_nsIHttpDataUsage_h__
#define __gen_nsIHttpDataUsage_h__


#ifndef __gen_nsISupports_h__
#include "nsISupports.h"
#endif

/* For IDL files that don't want to include root IDL files. */
#ifndef NS_NO_VTABLE
#define NS_NO_VTABLE
#endif

/* starting interface:    nsIHttpDataUsage */
#define NS_IHTTPDATAUSAGE_IID_STR "79dee3eb-9323-4d5c-b0a8-1baa18934d9e"

#define NS_IHTTPDATAUSAGE_IID \
  {0x79dee3eb, 0x9323, 0x4d5c, \
    { 0xb0, 0xa8, 0x1b, 0xaa, 0x18, 0x93, 0x4d, 0x9e }}

class NS_NO_VTABLE nsIHttpDataUsage : public nsISupports {
 public: 

  NS_DECLARE_STATIC_IID_ACCESSOR(NS_IHTTPDATAUSAGE_IID)

  /* readonly attribute unsigned long long ethernetBytesRead; */
  NS_IMETHOD GetEthernetBytesRead(uint64_t *aEthernetBytesRead) = 0;

  /* readonly attribute unsigned long long ethernetBytesWritten; */
  NS_IMETHOD GetEthernetBytesWritten(uint64_t *aEthernetBytesWritten) = 0;

  /* readonly attribute unsigned long long cellBytesRead; */
  NS_IMETHOD GetCellBytesRead(uint64_t *aCellBytesRead) = 0;

  /* readonly attribute unsigned long long cellBytesWritten; */
  NS_IMETHOD GetCellBytesWritten(uint64_t *aCellBytesWritten) = 0;

  /* void resetHttpDataUsage (); */
  NS_IMETHOD ResetHttpDataUsage(void) = 0;

};

  NS_DEFINE_STATIC_IID_ACCESSOR(nsIHttpDataUsage, NS_IHTTPDATAUSAGE_IID)

/* Use this macro when declaring classes that implement this interface. */
#define NS_DECL_NSIHTTPDATAUSAGE \
  NS_IMETHOD GetEthernetBytesRead(uint64_t *aEthernetBytesRead); \
  NS_IMETHOD GetEthernetBytesWritten(uint64_t *aEthernetBytesWritten); \
  NS_IMETHOD GetCellBytesRead(uint64_t *aCellBytesRead); \
  NS_IMETHOD GetCellBytesWritten(uint64_t *aCellBytesWritten); \
  NS_IMETHOD ResetHttpDataUsage(void); 

/* Use this macro to declare functions that forward the behavior of this interface to another object. */
#define NS_FORWARD_NSIHTTPDATAUSAGE(_to) \
  NS_IMETHOD GetEthernetBytesRead(uint64_t *aEthernetBytesRead) { return _to GetEthernetBytesRead(aEthernetBytesRead); } \
  NS_IMETHOD GetEthernetBytesWritten(uint64_t *aEthernetBytesWritten) { return _to GetEthernetBytesWritten(aEthernetBytesWritten); } \
  NS_IMETHOD GetCellBytesRead(uint64_t *aCellBytesRead) { return _to GetCellBytesRead(aCellBytesRead); } \
  NS_IMETHOD GetCellBytesWritten(uint64_t *aCellBytesWritten) { return _to GetCellBytesWritten(aCellBytesWritten); } \
  NS_IMETHOD ResetHttpDataUsage(void) { return _to ResetHttpDataUsage(); } 

/* Use this macro to declare functions that forward the behavior of this interface to another object in a safe way. */
#define NS_FORWARD_SAFE_NSIHTTPDATAUSAGE(_to) \
  NS_IMETHOD GetEthernetBytesRead(uint64_t *aEthernetBytesRead) { return !_to ? NS_ERROR_NULL_POINTER : _to->GetEthernetBytesRead(aEthernetBytesRead); } \
  NS_IMETHOD GetEthernetBytesWritten(uint64_t *aEthernetBytesWritten) { return !_to ? NS_ERROR_NULL_POINTER : _to->GetEthernetBytesWritten(aEthernetBytesWritten); } \
  NS_IMETHOD GetCellBytesRead(uint64_t *aCellBytesRead) { return !_to ? NS_ERROR_NULL_POINTER : _to->GetCellBytesRead(aCellBytesRead); } \
  NS_IMETHOD GetCellBytesWritten(uint64_t *aCellBytesWritten) { return !_to ? NS_ERROR_NULL_POINTER : _to->GetCellBytesWritten(aCellBytesWritten); } \
  NS_IMETHOD ResetHttpDataUsage(void) { return !_to ? NS_ERROR_NULL_POINTER : _to->ResetHttpDataUsage(); } 

#if 0
/* Use the code below as a template for the implementation class for this interface. */

/* Header file */
class nsHttpDataUsage : public nsIHttpDataUsage
{
public:
  NS_DECL_ISUPPORTS
  NS_DECL_NSIHTTPDATAUSAGE

  nsHttpDataUsage();

private:
  ~nsHttpDataUsage();

protected:
  /* additional members */
};

/* Implementation file */
NS_IMPL_ISUPPORTS1(nsHttpDataUsage, nsIHttpDataUsage)

nsHttpDataUsage::nsHttpDataUsage()
{
  /* member initializers and constructor code */
}

nsHttpDataUsage::~nsHttpDataUsage()
{
  /* destructor code */
}

/* readonly attribute unsigned long long ethernetBytesRead; */
NS_IMETHODIMP nsHttpDataUsage::GetEthernetBytesRead(uint64_t *aEthernetBytesRead)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* readonly attribute unsigned long long ethernetBytesWritten; */
NS_IMETHODIMP nsHttpDataUsage::GetEthernetBytesWritten(uint64_t *aEthernetBytesWritten)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* readonly attribute unsigned long long cellBytesRead; */
NS_IMETHODIMP nsHttpDataUsage::GetCellBytesRead(uint64_t *aCellBytesRead)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* readonly attribute unsigned long long cellBytesWritten; */
NS_IMETHODIMP nsHttpDataUsage::GetCellBytesWritten(uint64_t *aCellBytesWritten)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void resetHttpDataUsage (); */
NS_IMETHODIMP nsHttpDataUsage::ResetHttpDataUsage()
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* End of implementation class template. */
#endif


#endif /* __gen_nsIHttpDataUsage_h__ */
