/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */


#include "nsISupports.idl"

interface nsIDOMElement;
interface nsIDialogParamBlock;

[scriptable, uuid(17c9e659-2a9a-4a58-8a1c-d76bb58339c0)]
interface nsIDialogCreator : nsISupports
{
  const unsigned long UNKNOWN_DIALOG = 0;
  const unsigned long GENERIC_DIALOG = 1;
  const unsigned long SELECT_DIALOG  = 2;

  void openDialog(in unsigned long aType,
                  in ACString aName, in ACString aFeatures,
                  in nsIDialogParamBlock aArguments,
                  [optional] in nsIDOMElement aFrameElement);
};
